@@ -1,23 +1,11 @@
 /* This file is part of the KDE libraries
-   Copyright (C) 1998 Sven Radej <sven@lisa.exp.univie.ac.at>
-   Copyright (C) 2006 Dirk Mueller <mueller@kde.org>
-   Copyright (C) 2007 Flavio Castelli <flavio.castelli@gmail.com>
-   Copyright (C) 2008 Rafal Rzepecki <divided.mind@gmail.com>
-   Copyright (C) 2010 David Faure <faure@kde.org>
+   SPDX-FileCopyrightText: 1998 Sven Radej <sven@lisa.exp.univie.ac.at>
+   SPDX-FileCopyrightText: 2006 Dirk Mueller <mueller@kde.org>
+   SPDX-FileCopyrightText: 2007 Flavio Castelli <flavio.castelli@gmail.com>
+   SPDX-FileCopyrightText: 2008 Rafal Rzepecki <divided.mind@gmail.com>
+   SPDX-FileCopyrightText: 2010 David Faure <faure@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-only
 */
 
 // CHANGES:
@@ -215,7 +203,7 @@ KDirWatchPrivate::KDirWatchPrivate()
     m_inotify_fd = inotify_init();
 
     if (m_inotify_fd <= 0) {
-        qCDebug(KDIRWATCH) << "Can't use Inotify, kernel doesn't support it";
+        qCDebug(KDIRWATCH) << "Can't use Inotify, kernel doesn't support it:" << strerror(errno);
         supports_inotify = false;
     }
 
@@ -450,7 +438,7 @@ KDirWatchPrivate::Entry::~Entry()
  */
 void KDirWatchPrivate::Entry::propagate_dirty()
 {
-    Q_FOREACH (Entry *sub_entry, m_entries) {
+    for (Entry *sub_entry : qAsConst(m_entries)) {
         if (!sub_entry->dirty) {
             sub_entry->dirty = true;
             sub_entry->propagate_dirty();
@@ -565,7 +553,7 @@ QDebug operator<<(QDebug debug, const KDirWatchPrivate::Entry &entry)
     debug.space();
     if (!entry.m_entries.isEmpty()) {
         debug << ", nonexistent subentries:";
-        Q_FOREACH (KDirWatchPrivate::Entry *subEntry, entry.m_entries) {
+        for (KDirWatchPrivate::Entry *subEntry : qAsConst(entry.m_entries)) {
             debug << subEntry << subEntry->path;
         }
     }
@@ -583,7 +571,7 @@ KDirWatchPrivate::Entry *KDirWatchPrivate::entry(const QString &_path)
     QString path(_path);
 
     if (path.length() > 1 && path.endsWith(QLatin1Char('/'))) {
-        path.truncate(path.length() - 1);
+        path.chop(1);
     }
 
     EntryMap::Iterator it = m_mapEntries.find(path);
@@ -717,7 +705,7 @@ bool KDirWatchPrivate::useINotify(Entry *e)
     int mask = IN_DELETE | IN_DELETE_SELF | IN_CREATE | IN_MOVE | IN_MOVE_SELF | IN_DONT_FOLLOW | IN_MOVED_FROM | IN_MODIFY | IN_ATTRIB;
 
     if ((e->wd = inotify_add_watch(m_inotify_fd,
-                                   QFile::encodeName(e->path).data(), mask)) >= 0) {
+                                   QFile::encodeName(e->path).data(), mask)) != -1) {
         m_inotify_wd_to_entry.insert(e->wd, e);
         if (s_verboseDebug) {
             qCDebug(KDIRWATCH) << "inotify successfully used for monitoring" << e->path << "wd=" << e->wd;
@@ -725,7 +713,16 @@ bool KDirWatchPrivate::useINotify(Entry *e)
         return true;
     }
 
-    qCDebug(KDIRWATCH) << "inotify failed for monitoring" << e->path << ":" << strerror(errno);
+    if (errno == ENOSPC) {
+        // Inotify max_user_watches was reached (/proc/sys/fs/inotify/max_user_watches)
+        // See man inotify_add_watch, https://github.com/guard/listen/wiki/Increasing-the-amount-of-inotify-watchers
+        qCWarning(KDIRWATCH) << "inotify failed for monitoring" << e->path << "\n" <<
+                                "Because it reached its max_user_watches,\n" <<
+                                "you can increase the maximum number of file watches per user,\n"<<
+                                "by setting an appropriate fs.inotify.max_user_watches parameter in your /etc/sysctl.conf";
+    } else {
+        qCDebug(KDIRWATCH) << "inotify failed for monitoring" << e->path << ":" << strerror(errno) << " (errno:" << errno  <<  ")";
+    }
     return false;
 }
 #endif
@@ -740,7 +737,7 @@ bool KDirWatchPrivate::useQFSWatch(Entry *e)
         return true;
     }
 
-    qCDebug(KDIRWATCH) << "fsWatcher->addPath" << e->path;
+    //qCDebug(KDIRWATCH) << "fsWatcher->addPath" << e->path;
     if (!fsWatcher) {
         fsWatcher = new QFileSystemWatcher();
         connect(fsWatcher, SIGNAL(directoryChanged(QString)), this, SLOT(fswEventReceived(QString)));
@@ -799,7 +796,7 @@ void KDirWatchPrivate::addEntry(KDirWatch *instance, const QString &_path,
     }
 
     if (path.length() > 1 && path.endsWith(QLatin1Char('/'))) {
-        path.truncate(path.length() - 1);
+        path.chop(1);
     }
 
     EntryMap::Iterator it = m_mapEntries.find(path);
@@ -955,12 +952,18 @@ void KDirWatchPrivate::addWatch(Entry *e)
     switch (preferredMethod) {
 #if HAVE_FAM
     case KDirWatch::FAM: entryAdded = useFAM(e); break;
+#else
+    case KDirWatch::FAM: entryAdded = false; break;
 #endif
 #if HAVE_SYS_INOTIFY_H
     case KDirWatch::INotify: entryAdded = useINotify(e); break;
+#else
+    case KDirWatch::INotify: entryAdded = false; break;
 #endif
 #if HAVE_QFILESYSTEMWATCHER
     case KDirWatch::QFSWatch: entryAdded = useQFSWatch(e); break;
+#else
+    case KDirWatch::QFSWatch: entryAdded = false; break;
 #endif
     case KDirWatch::Stat: entryAdded = useStat(e); break;
     }
@@ -1107,7 +1110,7 @@ void KDirWatchPrivate::removeEntries(KDirWatch *instance)
         }
     }
 
-    Q_FOREACH (const QString &path, pathList) {
+    for (const QString &path : qAsConst(pathList)) {
         removeEntry(instance, path, nullptr);
     }
 
@@ -1365,7 +1368,7 @@ void KDirWatchPrivate::emitEvent(Entry *e, int event, const QString &fileName)
             path += QLatin1Char('/') + fileName;
 #elif defined(Q_OS_WIN)
             //current drive is passed instead of /
-            path += QDir::currentPath().left(2) + QLatin1Char('/') + fileName;
+            path += QDir::currentPath().leftRef(2) + QLatin1Char('/') + fileName;
 #endif
         }
     }
@@ -1402,28 +1405,16 @@ void KDirWatchPrivate::emitEvent(Entry *e, int event, const QString &fileName)
         // Emit the signals delayed, to avoid unexpected re-entrance from the slots (#220153)
 
         if (event & Deleted) {
-#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
-            QMetaObject::invokeMethod(c.instance, "setDeleted", Qt::QueuedConnection, Q_ARG(QString, path));
-#else
             QMetaObject::invokeMethod(c.instance, [c, path]() { c.instance->setDeleted(path); }, Qt::QueuedConnection);
-#endif
         }
 
         if (event & Created) {
-#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
-            QMetaObject::invokeMethod(c.instance, "setCreated", Qt::QueuedConnection, Q_ARG(QString, path));
-#else
             QMetaObject::invokeMethod(c.instance, [c, path]() { c.instance->setCreated(path); }, Qt::QueuedConnection);
-#endif
             // possible emit Change event after creation
         }
 
         if (event & Changed) {
-#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
-            QMetaObject::invokeMethod(c.instance, "setDirty", Qt::QueuedConnection, Q_ARG(QString, path));
-#else
             QMetaObject::invokeMethod(c.instance, [c, path]() { c.instance->setDirty(path); }, Qt::QueuedConnection);
-#endif
         }
     }
 }
@@ -1537,7 +1528,7 @@ void KDirWatchPrivate::slotRescan()
             // original changes.
             QStringList pendingFileChanges = entry->m_pendingFileChanges;
             pendingFileChanges.removeDuplicates();
-            Q_FOREACH (const QString &changedFilename, pendingFileChanges) {
+            for (const QString &changedFilename : qAsConst(pendingFileChanges)) {
                 if (s_verboseDebug) {
                     qCDebug(KDIRWATCH) << "processing pending file change for" << changedFilename;
                 }
@@ -1558,7 +1549,7 @@ void KDirWatchPrivate::slotRescan()
 
 #if HAVE_SYS_INOTIFY_H
     // Remove watch of parent of new created directories
-    Q_FOREACH (Entry *e, cList) {
+    for (Entry *e : qAsConst(cList)) {
         removeEntry(nullptr, e->parentDirectory(), e);
     }
 #endif
@@ -1719,7 +1710,7 @@ void KDirWatchPrivate::checkFAMEvent(FAMEvent *fe)
         } else if (e->isDir && !e->m_clients.empty()) {
             bool isDir = false;
             const QList<const Client *> clients = e->clientsForFileOrDir(tpath, &isDir);
-            Q_FOREACH (const Client *client, clients) {
+            for (const Client *client : clients) {
                 addEntry(client->instance, tpath, nullptr, isDir,
                          isDir ? client->m_watchModes : KDirWatch::WatchDirOnly);
             }
@@ -1779,7 +1770,7 @@ void KDirWatchPrivate::statistics()
             }
             if (!e->m_entries.isEmpty()) {
                 qCDebug(KDIRWATCH) << "    dependent entries:";
-                Q_FOREACH (Entry *d, e->m_entries) {
+                for (Entry *d : qAsConst(e->m_entries)) {
                     qCDebug(KDIRWATCH) << "      " << d << d->path << (d->m_status == NonExistent ? "NonExistent" : "EXISTS!!! ERROR!");
                     if (s_verboseDebug) {
                         Q_ASSERT(d->m_status == NonExistent); // it doesn't belong here otherwise
@@ -1819,7 +1810,7 @@ void KDirWatchPrivate::fswEventReceived(const QString &path)
             addWatch(e);
         } else if (e->isDir) {
             // Check if any file or dir was created under this directory, that we were waiting for
-            Q_FOREACH (Entry *sub_entry, e->m_entries) {
+            for (Entry *sub_entry : qAsConst(e->m_entries)) {
                 fswEventReceived(sub_entry->path); // recurse, to call scanEntry and see if something changed
             }
         } else {
@@ -1909,7 +1900,7 @@ QDateTime KDirWatch::ctime(const QString &_path) const
         return QDateTime();
     }
 
-    return QDateTime::fromTime_t(e->m_ctime);
+    return QDateTime::fromSecsSinceEpoch(e->m_ctime);
 }
 
 void KDirWatch::removeDir(const QString &_path)
