@@ -1,22 +1,11 @@
-/* This file is part of the KDE libraries
-    Copyright (C) 2000 Stephan Kulow <coolo@kde.org>
-                       David Faure <faure@kde.org>
-    Copyright (C) 2006 Kevin Ottens <ervin@kde.org>
+/*
+    This file is part of the KDE libraries
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    SPDX-FileCopyrightText: 2000 Stephan Kulow <coolo@kde.org>
+    SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2006 Kevin Ottens <ervin@kde.org>
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kjobuidelegate.h"
@@ -37,13 +26,23 @@ public:
     bool autoWarningHandling : 1;
 
     void connectJob(KJob *job);
-    void _k_result(KJob *job);
+    void _k_result();
 };
 
 KJobUiDelegate::KJobUiDelegate()
     : QObject(), d(new Private(this))
 {
+}
 
+KJobUiDelegate::KJobUiDelegate(Flags flags)
+    : QObject(), d(new Private(this))
+{
+    if (flags & AutoErrorHandlingEnabled) {
+        d->autoErrorHandling = true;
+    }
+    if (flags & AutoWarningHandlingEnabled) {
+        d->autoWarningHandling = true;
+    }
 }
 
 KJobUiDelegate::~KJobUiDelegate()
@@ -102,15 +101,12 @@ void KJobUiDelegate::slotWarning(KJob *job, const QString &plain,
 
 void KJobUiDelegate::connectJob(KJob *job)
 {
-    connect(job, SIGNAL(result(KJob*)),
-            this, SLOT(_k_result(KJob*)));
-
+    connect(job, &KJob::result, this, [this](){ d->_k_result();} );
     connect(job, &KJob::warning, this, &KJobUiDelegate::slotWarning);
 }
 
-void KJobUiDelegate::Private::_k_result(KJob *job2)
+void KJobUiDelegate::Private::_k_result()
 {
-    Q_UNUSED(job2)
     if (job->error() && autoErrorHandling) {
         q->showErrorMessage();
     }
